/*
 * Copyright (c) 2006-2007 CACE Technologies, Davis (California)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CACE Technologies nor the names of its 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#define CLIENT_DELAY_MS 5000

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <airpcap.h>
#include "ConfigFile.h"

//
// Run a command asynchronously inside the current shell
//
HANDLE CmdExec(const char * Cmd, char* args, BOOL Wait, DWORD* ProcessId)
{
	BOOL Res = FALSE;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	char commandstring[256];
	DWORD Flags = 0;
	
	// 
	// Configure the input parameters
	//
	memset(&si, 0, sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	si.wShowWindow = SW_SHOWMINIMIZED;

	// 
	// Execute the command with a call to the CreateProcess API call.
	//
	
	sprintf(commandstring, "%s %s", Cmd, args);
	
	Res = CreateProcess(NULL,
		commandstring,
		NULL,
		NULL,
		TRUE,
		Flags,
		NULL,
		NULL,
		&si,
		&pi);
	
	if(!Res)
	{
		return NULL;
	}
	

	if(Wait)
	{
		WaitForSingleObject(pi.hProcess, INFINITE);
	}

	*ProcessId = pi.dwProcessId;
	
	return pi.hProcess;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

int main()
{
	CHAR Ebuf[AIRPCAP_ERRBUF_SIZE];
	INT i;
	AirpcapDeviceDescription *AllDevs, *TmpDev;
	HANDLE ServerHandle, ClientHandle;
	DWORD ServerExitCode = 0, ClientExitCode = 0;
	ConfigFile cf("KismetConf.cfg");
	std::string s, s1;
	std::string ServerCmdString;
	DWORD ServerProcessId, TmpProcessId;
	char tb[64];


	//
	// Basic server command line
	//
	ServerCmdString = "kismet_server.exe --silent -f ./kismet.conf";

	//
	// parse logging state
	//
	s = cf.Value("KismetConf","EnableLogging").c_str();
	if(s != "yes")
	{
		ServerCmdString += std::string(" -n");
	}

	//
	// parse source type
	//
	s = cf.Value("KismetConf","LocalSourceType");

	if(s == "all" || s == "")
	{
		//
		// Get the AirPcap device list
		//
		if(!AirpcapGetDeviceList(&AllDevs, Ebuf))
		{
			printf("Unable to retrieve the device list: %s\n", Ebuf);
			return -1;
		}
		
		//
		// Scan the list and build the kismet_server command line
		//		
		for(TmpDev = AllDevs, i = 0; TmpDev; TmpDev = TmpDev->next)
		{
			// Skip the "any" interface
			if(std::string(TmpDev->Name).find("_any") == -1)
			{
				ServerCmdString = ServerCmdString + std::string(" -c airpcap,") + TmpDev->Name + std::string(",") + TmpDev->Name;
			}
		}

		if(s == "")
		{
			goto DoneLoading;
		}
	}
	else
	if(s == "specify")
	{
		s = cf.Value("KismetConf","LocalSource").c_str();
		
		if(s != "")
		{
			ServerCmdString = ServerCmdString + std::string(" -c airpcap,") + s + std::string(",") + s;
		}
	}

	//
	// Parse the drones list
	//
	for(i = 0; i < 100; i++)
	{
		s1 = std::string("Drone") + std::string(itoa(i, tb, 10));

		s = cf.Value("KismetConf", s1);
		if(s == "")
		{
			break;
		}

		ServerCmdString = ServerCmdString + std::string(" -c kismet_drone,") + s + std::string(",drone");
	}

DoneLoading:

	printf("COMMAND LINE:\n\t%s\n", ServerCmdString.c_str());

	//
	// Start the kismet server
	//
	printf("\n[Starting the Kismet server]\n");
	ServerHandle = CmdExec(ServerCmdString.c_str(), NULL, FALSE, &ServerProcessId);
	
	if(!ServerHandle)
	{
		// Server didn't start. Too bad.
		MessageBox(NULL, "Error starting the kismet server", "Error", MB_ICONERROR);
		return -1;
	}

	//
	// Give the server some time to go up
	//
	printf("\n[Waiting for the server to start]\n");

	Sleep(CLIENT_DELAY_MS);

	//
	// Start the kismet client, and wait until it's done
	//
	printf("\n[Starting the Kismet client]\n");
	ClientHandle = CmdExec("kismet_client.exe -f ./kismet.conf -u ./kismet_ui.conf", NULL, TRUE, &TmpProcessId);

	GetExitCodeProcess(ServerHandle, &ServerExitCode);

	if(ServerExitCode > 0 && ServerExitCode < 255 && ServerExitCode != STILL_ACTIVE)
	{
		MessageBox(NULL, "The kismet server couldn't start, make sure that at least one AirPcap adapter is present in the machine", "Error", MB_ICONERROR);
		return -1;
	}

	GetExitCodeProcess(ClientHandle, &ClientExitCode);

	if(ClientExitCode > 0 && ClientExitCode < 255 && ClientExitCode != STILL_ACTIVE)
	{
		MessageBox(NULL, "The kismet client couldn't start, please check your kismet.conf", "Error", MB_ICONERROR);
	}

	//
	// The client exited, kill the server before returning
	//
	s = std::string("kill.exe -s TERM ") + std::string(itoa(ServerProcessId, tb, 10));
	CmdExec(s.c_str(), NULL, TRUE, &TmpProcessId);

	return 0;
}
